@@ -1,32 +1,32 @@
 /*
-    Copyright 2005-2014 Intel Corporation.  All Rights Reserved.
+    Copyright (c) 2005-2019 Intel Corporation
 
-    This file is part of Threading Building Blocks. Threading Building Blocks is free software;
-    you can redistribute it and/or modify it under the terms of the GNU General Public License
-    version 2  as  published  by  the  Free Software Foundation.  Threading Building Blocks is
-    distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
-    implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
-    See  the GNU General Public License for more details.   You should have received a copy of
-    the  GNU General Public License along with Threading Building Blocks; if not, write to the
-    Free Software Foundation, Inc.,  51 Franklin St,  Fifth Floor,  Boston,  MA 02110-1301 USA
+    Licensed under the Apache License, Version 2.0 (the "License");
+    you may not use this file except in compliance with the License.
+    You may obtain a copy of the License at
 
-    As a special exception,  you may use this file  as part of a free software library without
-    restriction.  Specifically,  if other files instantiate templates  or use macros or inline
-    functions from this file, or you compile this file and link it with other files to produce
-    an executable,  this file does not by itself cause the resulting executable to be covered
-    by the GNU General Public License. This exception does not however invalidate any other
-    reasons why the executable file might be covered by the GNU General Public License.
+        http://www.apache.org/licenses/LICENSE-2.0
+
+    Unless required by applicable law or agreed to in writing, software
+    distributed under the License is distributed on an "AS IS" BASIS,
+    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+    See the License for the specific language governing permissions and
+    limitations under the License.
 */
 
 // TO DO: Add overlapping put / receive tests
 
 #include "harness.h"
+
+#if __TBB_CPF_BUILD
+#define TBB_DEPRECATED_FLOW_NODE_EXTRACTION 1
+#endif
+
 #include "tbb/flow_graph.h"
+#include "harness_checktype.h"
 #include "tbb/task_scheduler_init.h"
 #include "tbb/tick_count.h"
-#if TBB_PREVIEW_FLOW_GRAPH_FEATURES
 #include "harness_graph.h"
-#endif
 
 #include <cstdio>
 
@@ -79,7 +79,7 @@ struct parallel_put_get : NoAssign {
     parallel_put_get( tbb::flow::priority_queue_node<T> &q ) : my_q(q) {}
     void operator()(int tid) const {
         for ( int i = 0; i < N; i+=C ) {
-            int j_end = ( N < i + C ) ? N : i + C; 
+            int j_end = ( N < i + C ) ? N : i + C;
             // dump about C values into the Q
             for ( int j = i; j < j_end; ++j ) {
                 ASSERT( my_q.try_put( T (N*tid + j ) ) == true, NULL );
@@ -101,56 +101,60 @@ struct parallel_put_get : NoAssign {
 template< typename T >
 int test_reservation(int) {
     tbb::flow::graph g;
-    T bogus_value(-1);
 
     // Simple tests
     tbb::flow::priority_queue_node<T> q(g);
 
-    q.try_put(T(1));
-    q.try_put(T(2));
-    q.try_put(T(3));
-    g.wait_for_all();
+    {
 
-    T v=bogus_value, w=bogus_value;
-    ASSERT( q.try_reserve(v) == true, NULL );
-    ASSERT( v == T(3), NULL ); 
-    ASSERT( q.try_release() == true, NULL );
-    v = bogus_value;
-    g.wait_for_all();
-    ASSERT( q.try_reserve(v) == true, NULL );
-    ASSERT( v == T(3), NULL ); 
-    ASSERT( q.try_consume() == true, NULL );
-    v = bogus_value;
-    g.wait_for_all();
- 
-    ASSERT( q.try_get(v) == true, NULL );
-    ASSERT( v == T(2), NULL ); 
-    v = bogus_value;
-    g.wait_for_all();
-    
-    ASSERT( q.try_reserve(v) == true, NULL );
-    ASSERT( v == T(1), NULL ); 
-    ASSERT( q.try_reserve(w) == false, NULL );
-    ASSERT( w == bogus_value, NULL );
-    ASSERT( q.try_get(w) == false, NULL );
-    ASSERT( w == bogus_value, NULL );
-    ASSERT( q.try_release() == true, NULL );
-    v = bogus_value;
-    g.wait_for_all();
-    ASSERT( q.try_reserve(v) == true, NULL );
-    ASSERT( v == T(1), NULL ); 
-    ASSERT( q.try_consume() == true, NULL );
-    v = bogus_value;
-    g.wait_for_all();
-    ASSERT( q.try_get(v) == false, NULL );
+        T bogus_value(-1);
+
+        q.try_put(T(1));
+        q.try_put(T(2));
+        q.try_put(T(3));
+        g.wait_for_all();
+
+        T v=bogus_value, w=bogus_value;
+        ASSERT( q.try_reserve(v) == true, NULL );
+        ASSERT( v == T(3), NULL );
+        ASSERT( q.try_release() == true, NULL );
+        v = bogus_value;
+        g.wait_for_all();
+        ASSERT( q.try_reserve(v) == true, NULL );
+        ASSERT( v == T(3), NULL );
+        ASSERT( q.try_consume() == true, NULL );
+        v = bogus_value;
+        g.wait_for_all();
+
+        ASSERT( q.try_get(v) == true, NULL );
+        ASSERT( v == T(2), NULL );
+        v = bogus_value;
+        g.wait_for_all();
+
+        ASSERT( q.try_reserve(v) == true, NULL );
+        ASSERT( v == T(1), NULL );
+        ASSERT( q.try_reserve(w) == false, NULL );
+        ASSERT( w == bogus_value, NULL );
+        ASSERT( q.try_get(w) == false, NULL );
+        ASSERT( w == bogus_value, NULL );
+        ASSERT( q.try_release() == true, NULL );
+        v = bogus_value;
+        g.wait_for_all();
+        ASSERT( q.try_reserve(v) == true, NULL );
+        ASSERT( v == T(1), NULL );
+        ASSERT( q.try_consume() == true, NULL );
+        v = bogus_value;
+        g.wait_for_all();
+        ASSERT( q.try_get(v) == false, NULL );
+    }
     return 0;
 }
 
 //
 // Tests
 //
-// multilpe parallel senders, items in FIFO (relatively to sender) order
-// multilpe parallel senders, multiple parallel receivers, items in FIFO order (relative to sender/receiver) and all items received
+// multiple parallel senders, items in FIFO (relatively to sender) order
+// multiple parallel senders, multiple parallel receivers, items in FIFO order (relative to sender/receiver) and all items received
 //   * overlapped puts / gets
 //   * all puts finished before any getS
 //
@@ -227,7 +231,7 @@ int test_parallel(int num_threads) {
 // Tests
 //
 // Predecessors cannot be registered
-// Empty Q rejects item requests 
+// Empty Q rejects item requests
 // Single serial sender, items in FIFO order
 // Chained Qs ( 2 & 3 ), single sender, items at last Q in FIFO order
 //
@@ -328,20 +332,21 @@ int test_serial() {
     return 0;
 }
 
-int TestMain() { 
+int TestMain() {
     tbb::tick_count start = tbb::tick_count::now(), stop;
     for (int p = 2; p <= 4; ++p) {
         tbb::task_scheduler_init init(p);
         test_serial<int>();
         test_reservation<int>(p);
+        test_reservation<check_type<int> >(p);
         test_parallel<int>(p);
-    } 
+    }
     stop = tbb::tick_count::now();
     REMARK("Priority_Queue_Node Time=%6.6f\n", (stop-start).seconds());
-#if TBB_PREVIEW_FLOW_GRAPH_FEATURES
     REMARK("Testing resets\n");
     test_resets<int,tbb::flow::priority_queue_node<int> >();
     test_resets<float,tbb::flow::priority_queue_node<float> >();
+#if TBB_DEPRECATED_FLOW_NODE_EXTRACTION
     test_buffer_extract<tbb::flow::priority_queue_node<int> >().run_tests();
 #endif
     return Harness::Done;
