@@ -1,26 +1,21 @@
 /*
-    Copyright 2005-2014 Intel Corporation.  All Rights Reserved.
+    Copyright (c) 2005-2019 Intel Corporation
 
-    This file is part of Threading Building Blocks. Threading Building Blocks is free software;
-    you can redistribute it and/or modify it under the terms of the GNU General Public License
-    version 2  as  published  by  the  Free Software Foundation.  Threading Building Blocks is
-    distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
-    implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
-    See  the GNU General Public License for more details.   You should have received a copy of
-    the  GNU General Public License along with Threading Building Blocks; if not, write to the
-    Free Software Foundation, Inc.,  51 Franklin St,  Fifth Floor,  Boston,  MA 02110-1301 USA
+    Licensed under the Apache License, Version 2.0 (the "License");
+    you may not use this file except in compliance with the License.
+    You may obtain a copy of the License at
 
-    As a special exception,  you may use this file  as part of a free software library without
-    restriction.  Specifically,  if other files instantiate templates  or use macros or inline
-    functions from this file, or you compile this file and link it with other files to produce
-    an executable,  this file does not by itself cause the resulting executable to be covered
-    by the GNU General Public License. This exception does not however invalidate any other
-    reasons why the executable file might be covered by the GNU General Public License.
+        http://www.apache.org/licenses/LICENSE-2.0
+
+    Unless required by applicable law or agreed to in writing, software
+    distributed under the License is distributed on an "AS IS" BASIS,
+    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+    See the License for the specific language governing permissions and
+    limitations under the License.
 */
 
 #include "tbb/tick_count.h"
-#include "harness.h"
-#include <cstdio>
+#include "harness_assert.h"
 
 //! Assert that two times in seconds are very close.
 void AssertNear( double x, double y ) {
@@ -62,29 +57,24 @@ static void WaitForDuration( double duration ) {
         continue;
 }
 
+#include "harness.h"
+
 //! Test that average timer overhead is within acceptable limit.
 /** The 'tolerance' value inside the test specifies the limit. */
 void TestSimpleDelay( int ntrial, double duration, double tolerance ) {
-    double total_worktime = 0;
+    int error_count = 0;
+    double delta = 0;
     // Iteration -1 warms up the code cache.
     for( int trial=-1; trial<ntrial; ++trial ) {
-        tbb::tick_count t0 = tbb::tick_count::now();
+        tbb::tick_count t = tbb::tick_count::now();
         if( duration ) WaitForDuration(duration);
-        tbb::tick_count t1 = tbb::tick_count::now();
-        if( trial>=0 ) {
-            total_worktime += (t1-t0).seconds();
+        delta = (tbb::tick_count::now() - t).seconds() - duration;
+        if( trial>=0 && delta > tolerance ) {
+            error_count++;
         }
+        ASSERT(delta >= 0,"Delta is negative");
     }
-    // Compute average worktime and average delta
-    double worktime = total_worktime/ntrial;
-    double delta = worktime-duration;
-    REMARK("worktime=%g delta=%g tolerance=%g\n", worktime, delta, tolerance);
-
-    // Check that delta is acceptable
-    if( delta<0 )
-        REPORT("ERROR: delta=%g < 0\n",delta);
-    if( delta>tolerance )
-        REPORT("%s: delta=%g > %g=tolerance where duration=%g\n",delta>3*tolerance?"ERROR":"Warning",delta,tolerance,duration);
+    ASSERT(error_count < ntrial / 4, "The number of errors exceeded the threshold");
 }
 
 //------------------------------------------------------------------------
@@ -92,37 +82,71 @@ void TestSimpleDelay( int ntrial, double duration, double tolerance ) {
 //------------------------------------------------------------------------
 
 #include "tbb/atomic.h"
-static tbb::atomic<int> Counter;
-static volatile bool Flag;
+static tbb::atomic<int> Counter1, Counter2;
+static tbb::atomic<bool> Flag1, Flag2;
 static tbb::tick_count *tick_count_array;
+static double barrier_time;
 
 struct TickCountDifferenceBody {
+    TickCountDifferenceBody( int num_threads ) {
+        Counter1 = Counter2 = num_threads;
+        Flag1 = Flag2 = false;
+    }
     void operator()( int id ) const {
-        if( --Counter==0 ) Flag = true;
-        while( !Flag ) continue;
+        bool last = false;
+        // The first barrier.
+        if ( --Counter1 == 0 ) last = true;
+        while ( !last && !Flag1.load<tbb::acquire>() ) __TBB_Pause( 1 );
+        // Save a time stamp of the first barrier releasing.
         tick_count_array[id] = tbb::tick_count::now();
+
+        // The second barrier.
+        if ( --Counter2 == 0 ) Flag2.store<tbb::release>(true);
+        // The last thread should release threads from the first barrier after it reaches the second
+        // barrier to avoid a deadlock.
+        if ( last ) Flag1.store<tbb::release>(true);
+        // After the last thread releases threads from the first barrier it waits for a signal from
+        // the second barrier.
+        while ( !Flag2.load<tbb::acquire>() ) __TBB_Pause( 1 );
+
+        if ( last )
+            // We suppose that the barrier time is a time interval between the moment when the last
+            // thread reaches the first barrier and the moment when the same thread is released from
+            // the second barrier. This time is not accurate time of two barriers but it is
+            // guaranteed that it does not exceed it.
+            barrier_time = (tbb::tick_count::now() - tick_count_array[id]).seconds() / 2;
+    }
+    ~TickCountDifferenceBody() {
+        ASSERT( Counter1 == 0 && Counter2 == 0, NULL );
     }
 };
 
 //! Test that two tick_count values recorded on different threads can be meaningfully subtracted.
 void TestTickCountDifference( int n ) {
-    double tolerance = 3E-4;
+    const double tolerance = 3E-4;
     tick_count_array = new tbb::tick_count[n];
-    for( int trial=0; trial<10; ++trial ) {
-        Counter = n;
-        Flag = false;
-        NativeParallelFor( n, TickCountDifferenceBody() );
-        ASSERT( Counter==0, NULL );
-        for( int i=0; i<n; ++i )
-            for( int j=0; j<i; ++j ) {
-                double diff = (tick_count_array[i]-tick_count_array[j]).seconds();
-                if( diff<0 ) diff = -diff;
-                if( diff>tolerance ) {
-                    REPORT("%s: cross-thread tick_count difference = %g > %g = tolerance\n",
-                           diff>3*tolerance?"ERROR":"Warning",diff,tolerance);
-                }
+
+    int num_trials = 0;
+    tbb::tick_count start_time = tbb::tick_count::now();
+    do {
+        NativeParallelFor( n, TickCountDifferenceBody( n ) );
+        if ( barrier_time > tolerance )
+            // The machine seems to be oversubscribed so skip the test.
+            continue;
+        for ( int i = 0; i < n; ++i ) {
+            for ( int j = 0; j < i; ++j ) {
+                double diff = (tick_count_array[i] - tick_count_array[j]).seconds();
+                if ( diff < 0 ) diff = -diff;
+                if ( diff > tolerance )
+                    REPORT( "Warning: cross-thread tick_count difference = %g > %g = tolerance\n", diff, tolerance );
+                ASSERT( diff < 3 * tolerance, "Too big difference." );
             }
-    }
+        }
+        // During 5 seconds we are trying to get 10 successful trials.
+    } while ( ++num_trials < 10 && (tbb::tick_count::now() - start_time).seconds() < 5 );
+    REMARK( "Difference test time: %g sec\n", (tbb::tick_count::now() - start_time).seconds() );
+    // TODO: Find the cause of the machine high load, fix it and upgrade ASSERT_WARNING to ASSERT
+    ASSERT_WARNING( num_trials == 10, "The machine seems to be heavily oversubscribed, difference test was skipped." );
     delete[] tick_count_array;
 }
 
@@ -146,13 +170,17 @@ void TestResolution() {
     REMARK("avg_diff = %g ticks, max_diff = %g ticks\n", avg_diff, max_diff);
 }
 
-#include <tbb/compat/thread>
+#include "tbb/tbb_thread.h"
 
 int TestMain () {
+    // Increased tolerance for Virtual Machines
+    double tolerance_multiplier = Harness::GetEnv( "VIRTUAL_MACHINE" ) ? 50. : 1.;
+    REMARK( "tolerance_multiplier = %g \n", tolerance_multiplier );
+
     tbb::tick_count t0 = tbb::tick_count::now();
-    TestSimpleDelay(/*ntrial=*/1000000,/*duration=*/0,    /*tolerance=*/2E-6);
+    TestSimpleDelay(/*ntrial=*/1000000,/*duration=*/0,    /*tolerance=*/6E-6 * tolerance_multiplier);
     tbb::tick_count t1 = tbb::tick_count::now();
-    TestSimpleDelay(/*ntrial=*/10,     /*duration=*/0.125,/*tolerance=*/5E-6);
+    TestSimpleDelay(/*ntrial=*/1000,   /*duration=*/0.001,/*tolerance=*/15E-6 * tolerance_multiplier);
     tbb::tick_count t2 = tbb::tick_count::now();
     TestArithmetic(t0,t1,t2);
 
